/********************************************************************************
*                                                                               *
*                  Database query-result interface                              *
*                                                                               *
*********************************************************************************
* Copyright (C) 2002 by Mathew Robertson.       All Rights Reserved.            *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************/
#ifndef FXDATABASEQUERY_H
#define FXDATABASEQUERY_H

#ifndef FXBASEOBJECT_H
#include "FXBaseObject.h"
#endif
namespace FXEX {
class FXDatabaseInterface;

/**
 * Interface to data returned from database query
 */
class FXDatabaseQuery : public FXBaseObject {
  FXDECLARE(FXDatabaseQuery)

private:
  FXString             query;     // query performed
  FXDatabaseInterface *database;  // points to real database instance

protected:
  /// serialisation
  FXDatabaseQuery();

public:
  enum {
    ID_QUERY=FXBaseObject::ID_LAST,
    ID_MOVE_FIRST,
    ID_MOVE_NEXT,
    ID_MOVE_PREVIOUS,
    ID_MOVE_LAST,
    ID_MOVE_TO,
    ID_LAST
    };

public:
  long onMoveFirst(FXObject*,FXSelector,void*);
  long onMoveNext(FXObject*,FXSelector,void*);
  long onMovePrevious(FXObject*,FXSelector,void*);
  long onMoveLast(FXObject*,FXSelector,void*);
  long onMoveTo(FXObject*,FXSelector,void*);

public:
  /// build a query on a specific database
  FXDatabaseQuery(FXApp *a,FXDatabaseInterface *dbi=NULL,const FXString& qry="",FXObject *tgt=NULL,FXSelector sel=0);

  /// get database interface
  FXDatabaseInterface* getDatabase() const { return database; }

  /// set to new database
  void setDatabase(FXDatabaseInterface *dbi);
  
  /// get the last query
  FXString getQuery() const { return query; }

  /// set to new query
  void setQuery(const FXString& qry);

  /// execute query
  FXbool execute();

  /// return the current row
  void* fetchRow();

  /// return dataset
  void** fetchTable();

  /// FIXME provide access to the table, in cell form, row form (an array?), etc
//  operator[];

  /// Stream save and load
  virtual void save(FXStream& store) const;
  virtual void load(FXStream& store);

  /// dtor
  virtual ~FXDatabaseQuery();
  };

}  // namespace FXEX
#endif // FXDATABASEQUERY_H
