/***********************************-*- mode: c++; tab-width: 2 -*-*\
 *
 * NAME:     
 *   FXSplashWindow - Show a splash screen for app startup/about/etc
 *
 * AUTHOR:
 *   Daniel Gehriger (gehriger@linkcad.com)
 *
 * Updates:
 *   Mathew Robertson <mathew@users.sourceforge.net>
 *
 * Copyright (c) 1999 by Daniel Gehriger.  All Rights Reserved.
 *
 * PUPROSE:
 *   Display a splash window
 *
 * NOTE
 *
 * This library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU Library General Public   
 * License as published by the Free Software Foundation; either  
 * version 2 of the License, or (at your option) any later version.
 *                                                                 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.                 
 *                                                                  
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free       
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * $Id: FXSplashWindow.h,v 1.1 1999/12/12 23:05:21 dgehrige Exp $
 *
 * HISTORY:
 *        dgehrige - Dec, 10 1999: Created.
 *
 * #ident "@(#) $Id: FXSplashWindow.h,v 1.1 1999/12/12 23:05:21 dgehrige Exp $"
\*******************************************************************/
#ifndef FXSPLASHWINDOW_H
#define FXSPLASHWINDOW_H

#ifndef FXTOPWINDOW_H
#include <fox/FXTopWindow.h>
using namespace FX;
#endif
namespace FXEX {

/**
 * Application splash-window; something you might display to annoy the user
 *
 * Notes:
 * - just call execute() to show the window
 * - you are meant to derive from this and fill the contents of the window
 *   with your application info
 */

// splash window options
enum {
  SPLASH_NORAISE       = 0,                 // Dont keep splash window on top
  SPLASH_AUTORAISE     = 0x02000000,        // Keep splash window on top
  SPLASH_AUTODESTROY   = 0x04000000,        // Delete itself on timeout
  };

class FXAPI FXSplashWindow : public FXTopWindow  {
  FXDECLARE(FXSplashWindow)

protected:
  FXTimer* timer;     // the window timer
  FXuint   timeout;   // the amount of time specified

private:
  FXSplashWindow(const FXSplashWindow&);
  FXSplashWindow &operator=(const FXSplashWindow&);

protected:
  FXSplashWindow(){}

public:
  long onTimeout(FXObject*,FXSelector,void*);
  long onFocusOut(FXObject*,FXSelector,void*);

public:
  enum {
    ID_SPLASH=FXTopWindow::ID_LAST,
    ID_LAST
    };

public:
  // Create a splash window - default for 3 seconds delay
  FXSplashWindow(FXApp* a,const FXString& title="",FXint to=3000,FXuint opts=SPLASH_AUTORAISE|DECOR_BORDER,FXint w=0,FXint h=0,FXint hs=4,FXint vs=4);

  /// runs the splash window modally; saves calling create() then show()
  virtual FXuint execute(FXuint placement=PLACEMENT_SCREEN);

  /// show the splash dialog
  virtual void show();

  /// display splash dialog with a given placement
  virtual void show(FXuint placement);

  /// splash timeout (in miliseconds)
  void setTimeout(FXuint n) { timeout=n; }

  /// save to stream
  virtual void save(FXStream& store) const;

  /// load from stream
  virtual void load(FXStream& store);

  /// dtor
  virtual ~FXSplashWindow();
  };

} // namespace FXEX
#endif // FXSPLASHWINDOW_H
