/********************************************************************************
*                                                                               *
*                  Serial port I/O object                                       *
*                                                                               *
*********************************************************************************
* Copyright (C) 2002 by Mathew Robertson.   All Rights Reserved.                *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
********************************************************************************/
#include <fox/fxver.h>
#include <fox/xincs.h>
#include <fox/fxdefs.h>
#include <fox/FXStream.h>
#include <fox/FXString.h>
#include <fox/FXSize.h>
#include <fox/FXPoint.h>
#include <fox/FXRectangle.h>
#include <fox/FXRegistry.h>
#include <fox/FXApp.h>
#include <fox/FXObjectList.h>
using namespace FX;
#include "fxexdefs.h"
#include "FXSerialPort.h"
using namespace FXEX;
namespace FXEX {

/* Notes:
 * - to implement for Win32 use CreateFile(),OpenFile(),ReadFile(),CloseHandle()
 */

#ifdef WIN32
#  define errno GetLastError()
#endif

FXIMPLEMENT(FXSerialPort,FXIOHandle,NULL,0)

// device number specified ctor
FXSerialPort::FXSerialPort(FXApp *a,FXint no,FXObject *tgt,FXSelector sel) : FXIOHandle(a,tgt,sel){
  portNumber=no;
  if (no>0) code=FXIOStatusOk;
  }

// use an existing open device
FXSerialPort::FXSerialPort(FXInputHandle port,FXApp *a,FXObject *tgt,FXSelector sel) : FXIOHandle(port,a,tgt,sel){
  portNumber=-1;
  // FIXME need to query device for its name
  if (port != INVALID_HANDLE){
    state=FXIOStateOk;
    code=FXIOStatusOk;
    }
  }

// dtor
FXSerialPort::~FXSerialPort(){}

// save resources
void FXSerialPort::save(FXStream &store) const {
  FXIOHandle::save(store);
  store << portNumber;
  }

// load resources
void FXSerialPort::load(FXStream &store) {
  FXIOHandle::load(store);
  store >> portNumber;
  }

// set the port number
void FXSerialPort::setPortNumber(FXint no){
  if (state==FXIOStateNone) portNumber=no;
  }

// open the serial port
FXbool FXSerialPort::open(){
 
  // make sure we haven't already opened the serial port
  if (state != FXIOStateNone) return FALSE;

  // see what parent says about our handle
  if (!FXIOHandle::open()) return FALSE;

  // open the serial port
  FXString device;
#ifndef WIN32
  device="\\\\.\\com" + FXStringVal(portNumber+1);
  iohandle=::open(device.text(),O_RDWR|O_EXCL);
  if (iohandle==INVALID_HANDLE) { code=errno; return FALSE; }

#else
  device="/dev/ttyS" + FXStringVal(portNumber);
  iohandle=::CreateFile(device.text(),FILE_ALL_ACCESS,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
  if (iohandle==INVALID_HANDLE) { code=errno; return FALSE; }
  DCB devicestate;
  FXchar p;
  if (parity==ParityNone) p='N';
  if (parity==ParityOdd) p='O';
  if (parity==ParityEven) p='E';
  FXString params="baud="+ FXStringVal(speed) +" data="+ FXStringVal(databits) +" stop=" +
    FXStringVal(stopbits) +" parity="+ p;
  BuildCommDCB(params.text(),&devicestate);
  if (! SetCommState(iohandle,&devicestate)){ code=errno; return FALSE; }
#endif
  FXTRACE((100,"Serial port opened"));
  state=FXIOStateOk;
  return TRUE;
  }

/*
 * Create a serial port handle which is effectively a duplicate of this one
 */
FXSerialPort* FXSerialPort::duplicate(FXInputHandle newHandle){
  return dynamic_cast<FXSerialPort*>(FXIOHandle::duplicate(newHandle));
  }

// implement the instance creation
FXSerialPort* FXSerialPort::newInstance(FXInputHandle h){
  return new FXSerialPort(h,getApp(),target,message);
  }

}

