/********************************************************************************
*                                                                               *
*                  Socket Stream                                                *
*                                                                               *
*********************************************************************************
* Copyright (C) 2002 by Mathew Robertson.   All Rights Reserved.                *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************/
#include <fox/fxver.h>
#include <fox/xincs.h>
#include <fox/fxdefs.h>
#include <fox/FXStream.h>
#include <fox/FXString.h>
#include <fox/FXSize.h>
#include <fox/FXPoint.h>
#include <fox/FXRectangle.h>
#include <fox/FXRegistry.h>
#include <fox/FXApp.h>
#include <fox/FXObjectList.h>
using namespace FX;
#include "exincs.h"
#include "fxexdefs.h"
#include "FXSocket.h"
#include "FXSocketStream.h"
using namespace FXEX;
namespace FXEX {

// ctor
FXSocketStream::FXSocketStream(const FXObject *cont) : FXStream(cont){
  socket=NULL;
  opened=FALSE;
  }

// free up all resources
FXSocketStream::~FXSocketStream(){
  socket=(FXSocket*)-1;
  }

// Save to socket
// FIXME cannot save 'unsigned long' worth of data
void FXSocketStream::saveItems(const void *buf,unsigned long n){
  if(dir!=FXStreamSave){ fxerror("FXSocketStream::saveItems: wrong stream direction.\n"); }
  if(code==FXStreamOK){
    FXbool nonblock=socket->nonBlocking();
    if(nonblock) socket->nonBlocking(FALSE);   // set socket to blocking while we stream
    if(socket->write((FXuchar*)buf,n)!=(signed)n){code=FXStreamFull;}
    if(nonblock) socket->nonBlocking(TRUE);
    }
  }


// Load from socket
// FIXME cannot save 'unsigned long' worth of data
void FXSocketStream::loadItems(void *buf,unsigned long n){
  if(dir!=FXStreamLoad){ fxerror("FXSocketStream::loadItems: wrong stream direction.\n"); }
  if(code==FXStreamOK){
    FXint nonblock=socket->nonBlocking();
    if(nonblock) socket->nonBlocking(FALSE);
    if(socket->read((FXuchar*)buf,n)!=(signed)n){code=FXStreamEnd;}
    if(nonblock) socket->nonBlocking(TRUE);
    }
  }

// Try open socket stream
FXbool FXSocketStream::open(FXSocket *sock,FXStreamDirection save_or_load){

  // Stream should not yet be open
  if(dir!=FXStreamDead){ fxerror("FXSocketStream::open: stream is already open.\n"); }

  // make sure socket is good, and opened
  if(!sock) { fxerror("FXSocketStream::open: must provide a real socket.\n"); }
  FXint state=sock->getState();
  if(state==FXIOStateUnconnected){
    if(!sock->open()) {code=FXStreamFailure;return FALSE;}
    opened=TRUE;
    state=sock->getState();
    }

  // Open for read
  if(save_or_load==FXStreamLoad){
    if(state!=FXIOStateConnected && state!=FXIOStateAccepted){code=FXStreamNoRead;return FALSE;}
    }

  // Open for write
  else{
    if(state!=FXIOStateConnected && state!=FXIOStateAccepted){code=FXStreamNoWrite;return FALSE;}
    }

  // Do the generic book-keeping
  socket=sock;
  return FXStream::open(save_or_load);
  }


// Close socket stream
FXbool FXSocketStream::close(){
  if(socket && opened){
    socket->close();
    socket=NULL;
    opened=FALSE;
    }
  return FXStream::close();
  }

}

