/***********************************-*- mode: c++; tab-width: 2 -*-*\
 *
 * NAME:     
 *   FXSplashWindow - File dialog using native dialog
 *
 * AUTHOR:
 *   Daniel Gehriger (gehriger@linkcad.com)
 *
 * Copyright (c) 1999 by Daniel Gehriger.  All Rights Reserved.
 *
 * PUPROSE:
 *   Display a splash window
 *
 * NOTE
 *
 * This library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU Library General Public   
 * License as published by the Free Software Foundation; either  
 * version 2 of the License, or (at your option) any later version.
 *                                                                 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.                 
 *                                                                  
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free       
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * $Id: FXSplashWindow.cpp,v 1.1 1999/12/12 23:05:22 dgehrige Exp $
 *
 * HISTORY:
 *        dgehrige - Dec, 10 1999: Created.
 *
\*******************************************************************/
#include <fox/fxver.h>
#include <fox/xincs.h>
#include <fox/fxdefs.h>
#include <fox/FXStream.h>
#include <fox/FXString.h>
#include <fox/FXSize.h>
#include <fox/FXPoint.h>
#include <fox/FXRectangle.h>
#include <fox/FXRegistry.h>
#include <fox/FXApp.h>
using namespace FX;
#include "FXSplashWindow.h"
using namespace FXEX;
namespace FXEX {

#define SPLASHWINDOW_MASK (SPLASH_AUTORAISE)

FXDEFMAP(FXSplashWindow) FXSplashWindowMap[]={
  FXMAPFUNC(SEL_FOCUSOUT,0,FXSplashWindow::onFocusOut),
  FXMAPFUNC(SEL_TIMEOUT,FXSplashWindow::ID_SPLASH, FXSplashWindow::onTimeout),
  };
FXIMPLEMENT(FXSplashWindow,FXTopWindow,FXSplashWindowMap,ARRAYNUMBER(FXSplashWindowMap))

// ctor
FXSplashWindow::FXSplashWindow(FXApp *a,const FXString &title,FXint to,FXuint opts,FXint w,FXint h,FXint hs,FXint vs) : FXTopWindow(a,title,NULL,NULL,opts,0,0,w,h,0,0,0,0,hs,vs),timer(NULL),timeout(to){}


// remove timeout on destruction
FXSplashWindow::~FXSplashWindow() {
  if (timer) timer=getApp()->removeTimeout(timer);
  }

// save to stream
void FXSplashWindow::save(FXStream& store) const {
  FXTopWindow::save(store);
//FIXME  store << timer;
  store << timeout;
  }

// load from stream
void FXSplashWindow::load(FXStream& store) {
  FXTopWindow::load(store);
//FIXME  store >> timer;
  store >> timeout;
  }

// run splashwindow modally
FXuint FXSplashWindow::execute(FXuint placement){
  create();
  show(placement);
  return getApp()->runModalFor(this);
  }

// Show splash window
void FXSplashWindow::show() {
  if (timer) timer=getApp()->removeTimeout(timer);
  timer=getApp()->addTimeout(timeout,this,ID_SPLASH);
  FXTopWindow::show();
  }

// Show splash window with a given placement
void FXSplashWindow::show(FXuint placement) {
  if (timer) timer=getApp()->removeTimeout(timer);
  timer=getApp()->addTimeout(timeout,this,ID_SPLASH);
  FXTopWindow::show(placement);
  }

// timeout expired -> stop showing it
long FXSplashWindow::onTimeout(FXObject*, FXSelector, void*) {
  hide();
  if (options & SPLASH_AUTODESTROY) delete this;
  return 1;
  }

// Lost focus -> raise window
long FXSplashWindow::onFocusOut(FXObject*, FXSelector, void*) {
  if (options & SPLASH_AUTORAISE) {
    raise();
    return 1;
    }
  return 0;
  }

}

