/********************************************************************************
*                                                                               *
*                   Base type for threadable objects                            *
*                                                                               *
*********************************************************************************
* Copyright (C) 2002 by Mathew Robertson.   All Rights Reserved.                *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************/
#include <fox/fxver.h>
#include <fox/xincs.h>
#include <fox/fxdefs.h>
#include <fox/FXString.h>
#include <fox/FXStream.h>
#include <fox/FXSize.h>
#include <fox/FXPoint.h>
#include <fox/FXRectangle.h>
#include <fox/FXRegistry.h>
#include <fox/FXApp.h>
using namespace FX;
#include "fxexdefs.h"
#include "FXMutex.h"
#include "FXThreadEvent.h"
#include "FXThreadedObject.h"
using namespace FXEX;
namespace FXEX {

// map
FXIMPLEMENT_ABSTRACT(FXThreadedObject,FXObject,NULL,0)

// Base threadable object
FXThreadedObject::FXThreadedObject(FXObject *tgt,FXSelector sel) : FXObject() {
  target=tgt;
  message=sel;
  event.setTarget(tgt);
  event.setSelector(sel);
  }

// cleanup resources
FXThreadedObject::~FXThreadedObject(){
  target=(FXObject*)-1;
  }

// return application object
FXApp* FXThreadedObject::getApp(){
  return FXApp::instance();
  }

// set target
void FXThreadedObject::setTarget(FXObject *tgt){
  mutex.lock();
  target=tgt;
  event.setTarget(tgt);
  mutex.unlock();
  }

// get target
FXObject* FXThreadedObject::getTarget(){
  mutex.lock();
  FXObject *tgt=target;
  mutex.unlock();
  return tgt;
  }

// set the selector
void FXThreadedObject::setSelector(FXSelector sel){
  mutex.lock();
  message=sel;
  event.setSelector(sel);
  mutex.unlock();
  }

// get the selector
FXSelector FXThreadedObject::getSelector(){
  mutex.lock();
  FXSelector sel=message;
  mutex.unlock();
  return sel;
  }

// signal the target thread, using the SEL_THREAD message
void FXThreadedObject::signal(){
  event.signal();
  }

// specialised seltype message
void FXThreadedObject::signal(FXuint seltype){
  event.signal(seltype);
  }

}

