#include "fxrotatablefont.h"

// Event Handler Object
class AppWindow : public FXMainWindow
{ // Macro for class hierarchy declarations
  FXDECLARE(AppWindow)

	private:
    FXLabel      *textTop;
    FXRotatableFont    *font;
    FXCanvas     *canvas; // Canvas to draw on
    FXImage      *saveImage;
    FXfloat      A, B, C, D, A1, B1, C1, D1;
    FXint        GphWid, GphHt;
    FXint        dirty; // Canvas has been painted?
    FXColor      ForeGround, BackGround;

  protected:
    AppWindow(){}

  public: // regular methods
    void initializeApp(void);
  public: // event methods
    long onCmdClearCanvas(FXObject*, FXSelector, void*);
    long onUpdClear(FXObject*, FXSelector, void*);
    long onCmdRePaint(FXObject *sender, FXSelector, void *ptr);
    long onCmdGetFont(FXObject*, FXSelector, void*);
    long onCmdDrawRotatedText(FXObject*, FXSelector, void*);

  public:
    enum
    { ID_CANVAS=FXMainWindow::ID_LAST,
      ID_CLEAR,
      ID_FONT_DIALOG,
	  ID_DRAW_ROTATED_TEXT,
      ID_TEXT,
      ID_LAST
    };

  public:
    AppWindow(FXApp* a);
    virtual ~AppWindow();
    virtual void create();
};
// End Header Part

FXDEFMAP(AppWindow)
  AppWindowMap[] =
  { //________Message_Type______________________ID____________________Message_Handler_______
    FXMAPFUNC(SEL_COMMAND,           AppWindow::ID_CLEAR,             AppWindow::onCmdClearCanvas),
    FXMAPFUNC(SEL_UPDATE,            AppWindow::ID_CLEAR,             AppWindow::onUpdClear),
    FXMAPFUNC(SEL_PAINT,             AppWindow::ID_CANVAS,            AppWindow::onCmdRePaint),
    FXMAPFUNC(SEL_COMMAND,           AppWindow::ID_FONT_DIALOG,       AppWindow::onCmdGetFont),
	FXMAPFUNC(SEL_COMMAND,			 AppWindow::ID_DRAW_ROTATED_TEXT, AppWindow::onCmdDrawRotatedText),
  };

FXIMPLEMENT(AppWindow, FXMainWindow, AppWindowMap, ARRAYNUMBER(AppWindowMap))

// Construct AppWindow
AppWindow::AppWindow(FXApp *a):FXMainWindow(a, "XFontGUI", NULL, NULL, DECOR_ALL,
                                            0, 0, 0, 0)
{ GphWid = 600; GphHt = 400;
  //Create a Main FXVerticalFrame Widget
  FXVerticalFrame *mainFrame = new FXVerticalFrame(this,
    FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_TOP | LAYOUT_LEFT,
    0,0,0,0); //, 10,10,10,10);

  // Top horizontal frame will contain 2 vertical frames
  FXHorizontalFrame *topHorizFrame = new FXHorizontalFrame(mainFrame,
    LAYOUT_SIDE_TOP | LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0);

  // LEFT vertical frame to contain the canvas
  FXVerticalFrame *canvasFrame = new FXVerticalFrame(topHorizFrame,
    FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_TOP | LAYOUT_LEFT,
    0,0,0,0, 10,10,10,10);

  // Label above the canvas
  new FXLabel(canvasFrame, "Test X Font Properties", NULL, JUSTIFY_CENTER_X | LAYOUT_FILL_X);

  // Horizontal divider line
  new FXHorizontalSeparator(canvasFrame, SEPARATOR_GROOVE | LAYOUT_FILL_X);

  // Drawing canvas
  canvas = new FXCanvas(canvasFrame, this, ID_CANVAS,
    FRAME_SUNKEN | LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT | LAYOUT_TOP | LAYOUT_LEFT,
    0,0,GphWid,GphHt);

  // RIGHT frame for the test group box & buttons
  FXVerticalFrame *inputFrame = new FXVerticalFrame(topHorizFrame,
    FRAME_SUNKEN | LAYOUT_FILL_Y | LAYOUT_TOP | LAYOUT_LEFT,
    0,0,0,0, 10,10,10,10);

  FXGroupBox *FontButtons = new FXGroupBox(inputFrame, "Font Tests",
      GROUPBOX_NORMAL | FRAME_RIDGE | LAYOUT_FILL_X, 0,0,0,0, 10,10,5,5);
  // Button to draw Circle text
  // Button to draw rotated text
  new FXButton(FontButtons, "Rotated", NULL, this, ID_DRAW_ROTATED_TEXT,
               FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT,
               0,0,0,0, 10,10,5,5);
  // Button to draw Shear text
  // Button to draw Anamorphic text
  // Button to draw X Mirror text
  // Button to draw Y Mirror text
  // Button to draw Complex Matrix Multipled text

  FXGroupBox *SelectFont = new FXGroupBox(inputFrame, "Change Font",
      GROUPBOX_NORMAL | FRAME_RIDGE | LAYOUT_FILL_X, 0,0,0,0, 10,10,5,5);
  // Select Font Dialog Box
  new FXButton(SelectFont, "Select Font", NULL, this, ID_FONT_DIALOG,
               FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT,
               0,0,0,0, 10,10,5,5);

  // Button to clear
  new FXButton(SelectFont, "&Clear", NULL, this, ID_CLEAR,
               FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT,
               0,0,0,0, 10,10,5,5);

  // Exit button
  new FXButton(inputFrame, "&Exit", NULL, getApp(), FXApp::ID_QUIT,
    FRAME_THICK | FRAME_RAISED | LAYOUT_FILL_X | LAYOUT_BOTTOM | LAYOUT_LEFT,
    0,0,0,0, 10,10,5,5);

  //Bottom vertical frame contains text output window
  FXVerticalFrame *bottomFrame = new FXVerticalFrame(mainFrame, //bottomHorizFrame,
    FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FIX_HEIGHT | LAYOUT_TOP | LAYOUT_LEFT,
    0,0,0,145, 10,10,10,10);

  textTop = new FXLabel(bottomFrame, "X Font String", NULL,
    JUSTIFY_CENTER_X | LAYOUT_FILL_X, 0,0,0,0, 0,0,3,3);

  saveImage = new FXImage(getApp(), NULL, IMAGE_KEEP | IMAGE_OWNED, GphWid, GphHt);
  //textFont = new FXRotatableFont(getApp(), "helvetica", 12);

  cerr<<"constructing font"<<endl;
  font=new FXRotatableFont(getApp(),"bookman",16,90);
  cerr<<"done"<<endl;

  //TestCharFont=NULL;
} // AppWindow::AppWindow

// Clean up
AppWindow::~AppWindow()
{ delete saveImage;
  if (font) delete font;
} // AppWindow::~AppWindow

// Create and initialize
void AppWindow::create()
{ // Create the windows
  FXMainWindow::create();
  saveImage->create();
  cerr<<"creating font"<<endl;
  font->create();
  cerr<<"done"<<endl;
  // Make the main window appear
  show();
  initializeApp();
} // AppWindow::create

void AppWindow::initializeApp(void)
{ BackGround = FXRGB(255,255,255); //WHITE;
  ForeGround = FXRGB(255,0,0); //RED;
  FXDCWindow dc(saveImage);
  dc.setForeground(BackGround);
  dc.fillRectangle(0, 0, saveImage->getWidth(), saveImage->getHeight());
  textTop->setBackColor(BackGround);
  dirty = 0;
} // AppWindow::initialize

// Handle the clear message
long AppWindow::onCmdClearCanvas(FXObject*, FXSelector, void*)
{ FXDCWindow dc(saveImage);
  dc.setForeground(BackGround);
  dc.fillRectangle(0, 0, saveImage->getWidth(), saveImage->getHeight());
  FXDCWindow sdc(canvas);
  sdc.drawImage(saveImage, 0, 0);
  dirty = 0;
  return 1;
} // AppWindow::onCmdClear

// Update the clear button -- As you can see the Scribble Demo was handy
long AppWindow::onUpdClear(FXObject* sender, FXSelector, void*)
{ if(dirty)
    sender->handle(this, MKUINT(FXWindow::ID_ENABLE, SEL_COMMAND), NULL);
  else
    sender->handle(this, MKUINT(FXWindow::ID_DISABLE, SEL_COMMAND), NULL);
  return 1;
} // AppWindow::onUpdClear

// Repaint the canvas
long AppWindow::onCmdRePaint(FXObject *sender,FXSelector,void *ptr)
{ FXCanvas *canvas=(FXCanvas*) sender;
  FXEvent *ev=(FXEvent*)ptr;
  FXDCWindow dc(canvas, ev);
  dc.setForeground(BackGround);
  dc.fillRectangle(ev->rect.x, ev->rect.y, ev->rect.w, ev->rect.h);
  if (dirty == 1)
  { dc.drawImage(saveImage, 0, 0);
  }
  return 1;
} // AppWindow::onCmdRePaint

// Change font using FXFont Dialog box
long AppWindow::onCmdGetFont(FXObject*, FXSelector, void*)
{ 
	FXFontDialog fontdlg(this, "Change Font", DECOR_BORDER | DECOR_TITLE);
  	FXFontDesc fontdesc;

  	if(fontdlg.execute())
  	{ 
		fontdlg.getFontSelection(fontdesc);
		if (font) delete font;
    	font = new FXRotatableFont(getApp(), fontdesc, 90);
    	font->create();
  		return 1;
  	}
  	return 0;
}



long AppWindow::onCmdDrawRotatedText(FXObject*,FXSelector,void*)
{
	FXDCWindow dc(saveImage);
	dc.setForeground(ForeGround);
	font->display(dc,"Initial font angle",320,340);
	for (float angle=0; angle<360; angle+=15)
	{
		font->changeAngle(angle);
		font->display(dc,"Rotated text!",160,170);
	}
	font->changeAngle(90);
  	FXDCWindow sdc(canvas);
  	sdc.drawImage(saveImage, 0, 0);
	dirty=1;
	return 1;
}


// Here we begin
int main(FXint argc, char *argv[])
{ // Make application
  FXApp* application = new FXApp("My Window","Test");
  // Start app
  application->init(argc, argv);
  // Application window
  new AppWindow(application);
  // Create the application's windows
  application->create();
  // Run the application
  application->run();
  return 0;
} // main
